# -*- coding: utf-8 -*-

# pylint: disable=no-name-in-module

# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------

"""Quit application dialog."""

from qtpy.QtCore import QSize, Qt
from qtpy.QtSvg import QSvgWidget
from qtpy.QtWidgets import QDialogButtonBox, QHBoxLayout, QLabel, QListWidgetItem, QVBoxLayout
from anaconda_navigator.config import CONF
from anaconda_navigator.static import images
from anaconda_navigator.utils.styles import SASS_VARIABLES
from anaconda_navigator.widgets import (
    ButtonDanger, ButtonNormal, ButtonPrimary, CheckBoxBase, FrameBase, LabelBase, SpacerHorizontal, SpacerVertical,
)
from anaconda_navigator.widgets.dialogs import DialogBase
from anaconda_navigator.widgets.lists import ListWidgetBase


class QuitApplicationDialog(DialogBase):
    """Quit application confirmation dialog."""
    def __init__(self, *args, **kwargs):
        """Quit application confirmation dialog."""
        self.config = kwargs.pop('config', CONF)
        super().__init__(*args, **kwargs)
        self.widget_icon = QSvgWidget(images.ANACONDA_LOGO)
        self.label_about = QLabel('Quit Anaconda Navigator?')
        self.button_ok = ButtonPrimary('Yes')
        self.button_cancel = ButtonNormal('No')
        self.buttonbox = QDialogButtonBox(Qt.Horizontal)
        self.checkbox = CheckBoxBase("Don't show again")

        # Widgets setup
        self.setWindowTitle('Quit application')
        self.widget_icon.setFixedSize(QSize(100, 100))

        # Layouts
        h_layout = QHBoxLayout()
        h_layout.addWidget(self.widget_icon, 0, Qt.AlignTop)
        h_layout.addWidget(SpacerHorizontal())
        h_layout.addWidget(self.label_about)

        layout_buttons = QHBoxLayout()
        layout_buttons.addStretch()
        layout_buttons.addWidget(self.button_cancel)
        layout_buttons.addWidget(SpacerHorizontal())
        layout_buttons.addWidget(self.button_ok)

        main_layout = QVBoxLayout()
        main_layout.addLayout(h_layout)
        main_layout.addWidget(self.checkbox, 0, Qt.AlignRight)
        main_layout.addWidget(SpacerVertical())
        main_layout.addWidget(SpacerVertical())
        main_layout.addLayout(layout_buttons)
        self.setLayout(main_layout)

        # Signals
        self.button_ok.clicked.connect(self.accept)
        self.button_cancel.clicked.connect(self.reject)

        # Setup
        self.update_style_sheet()
        self.setup()
        self.button_cancel.setFocus()

    def setup(self):
        """Setup widget content."""
        # Widget setup
        if self.config.get('main', 'hide_quit_dialog'):
            hide_dialog = Qt.Checked
        else:
            hide_dialog = Qt.Unchecked
        self.checkbox.setChecked(hide_dialog)

    def accept(self):
        """
        Qt overide.

        Update the configuration preferences.
        """
        hide_dialog = self.checkbox.checkState() == Qt.Checked
        self.config.set('main', 'hide_quit_dialog', hide_dialog)
        super().accept()


class QuitBusyDialog(QuitApplicationDialog):
    """Dialog for quiting while navigator is busy."""
    def __init__(self, *args, **kwargs):
        """Quit application confirmation dialog."""
        super().__init__(*args, **kwargs)
        self.label_about.setText('Anaconda Navigator is still busy.<br><br>Do you want to quit?')
        self.setWindowTitle('Quit application')
        self.checkbox.setVisible(False)


class FrameRunningApps(FrameBase):
    """Main widget for the list items."""


class ListRunningApps(ListWidgetBase):
    """List of running apps."""

    def setup_item(self, item):
        """Override."""


class ListItemRunningApp(QListWidgetItem):
    """List item representing a running application."""
    def __init__(self, package):
        """List item representing a running application."""
        super().__init__()
        self.widget = FrameRunningApps()
        self.package = package
        self.label = LabelBase(package)
        self.checkbox = CheckBoxBase()

        layout_frame = QHBoxLayout()
        layout_frame.addWidget(self.checkbox)
        layout_frame.addWidget(self.label)
        layout_frame.addStretch()
        self.widget.setLayout(layout_frame)
        self.setSizeHint(self.widget_size())

    def set_checked(self, value):
        """Set the check state for the checkbox in the list item."""
        self.checkbox.setChecked(value)

    def checked(self):
        """Return True if checked otherwise return False."""
        return self.checkbox.isChecked()

    @staticmethod
    def set_loading(item):
        """Override."""

    def text(self):
        """Qt override."""
        return self.label.text()

    @staticmethod
    def set_selected(item):
        """Override."""

    @staticmethod
    def widget_size():
        """Return the size defined in the SASS file."""
        return QSize(SASS_VARIABLES.WIDGET_RUNNING_APPS_TOTAL_WIDTH, SASS_VARIABLES.WIDGET_RUNNING_APPS_TOTAL_HEIGHT)


class QuitRunningAppsDialog(DialogBase):  # pylint: disable=too-many-instance-attributes
    """Dialog for closing running apps if quiting navigator."""
    def __init__(self, *args, **kwargs):
        """Dialog for closing running apps if quiting navigator."""
        self.running_processes = kwargs.pop('running_processes', [])
        self.config = kwargs.pop('config', CONF)
        super().__init__(*args, **kwargs)

        self.list = ListRunningApps(self)
        self.label_about = QLabel(
            'There are some applications running. Please select \nthe applications you want to close on quit:'
        )
        self.button_close = ButtonDanger('Quit')
        self.button_cancel = ButtonNormal('Cancel')
        self.buttonbox = QDialogButtonBox(Qt.Horizontal)
        self.checkbox = CheckBoxBase('Don\'t show again')

        # Widget setup
        self.setWindowTitle('Close running applications')

        # Layouts
        layout_buttons = QHBoxLayout()
        layout_buttons.addStretch()
        layout_buttons.addWidget(self.button_cancel)
        layout_buttons.addWidget(SpacerHorizontal())
        layout_buttons.addWidget(self.button_close)

        main_layout = QVBoxLayout()
        main_layout.addWidget(self.label_about)
        main_layout.addWidget(SpacerVertical())
        main_layout.addWidget(self.list, 0, Qt.AlignCenter)
        main_layout.addWidget(SpacerVertical())
        main_layout.addWidget(self.checkbox, 0, Qt.AlignRight)
        main_layout.addWidget(SpacerVertical())
        main_layout.addWidget(SpacerVertical())
        main_layout.addLayout(layout_buttons)
        self.setLayout(main_layout)

        # Signals
        self.button_close.clicked.connect(self.accept)
        self.button_cancel.clicked.connect(self.reject)

        # Setup
        self.update_style_sheet()
        self.setup()
        self.button_cancel.setFocus()

    def setup(self):
        """Setup widget content."""
        # Widget setup
        packages = sorted({
            package.package
            for package in self.running_processes
        })
        checked_packages = self.config.get('main', 'running_apps_to_close')
        for package in packages:
            item = ListItemRunningApp(package)
            item.set_checked(package in checked_packages)
            self.list.addItem(item)

        if self.config.get('main', 'hide_running_apps_dialog'):
            hide_dialog = Qt.Checked
        else:
            hide_dialog = Qt.Unchecked
        self.checkbox.setChecked(hide_dialog)

    def accept(self):
        """
        Qt overide.

        Update the configuration preferences.
        """
        original_checked_packages = set(self.config.get('main', 'running_apps_to_close'))
        hide_dialog = self.checkbox.checkState() == Qt.Checked
        checked_packages = {
            item.text()
            for item in self.list.items()
            if item.checked()
        }
        packages = {
            item.text()
            for item in self.list.items()
        }
        delta = original_checked_packages - packages
        new_packages = sorted(list(checked_packages.union(delta)))
        self.config.set('main', 'running_apps_to_close', new_packages)
        self.config.set('main', 'hide_running_apps_dialog', hide_dialog)
        super().accept()


class ClosePackageManagerDialog(QuitApplicationDialog):
    """Confirm application quit if package manager is running."""
    def __init__(self, *args, **kwargs):
        """Confirm application quit if package manager is running."""
        super().__init__(*args, **kwargs)

        self.label_about.setText('Conda is still busy.\n\nDo you want to cancel the process?')
        self.setWindowTitle('Cancel Process')
        self.checkbox.setVisible(False)



#  tests for anaconda-navigator-2.5.2-py311haa95532_0 (this is a generated file);
print('===== testing package: anaconda-navigator-2.5.2-py311haa95532_0 =====');
print('running run_test.py');
#  --- run_test.py (begin) ---
import anaconda_navigator

print('anaconda_navigator.__version__: %s' % anaconda_navigator.__version__)
assert anaconda_navigator.__version__ == '2.5.2'
#  --- run_test.py (end) ---

print('===== anaconda-navigator-2.5.2-py311haa95532_0 OK =====');
print("import: 'anaconda_navigator'")
import anaconda_navigator




# -*- coding: utf8 -*-

"""Notification utilities."""

from __future__ import annotations

__all__ = ('Notification', 'NotificationListener', 'NotificationQueue', 'NOTIFICATION_QUEUE')

import collections.abc
import typing

from qtpy.QtCore import QObject, Signal  # pylint: disable=no-name-in-module

if typing.TYPE_CHECKING:
    import typing_extensions


RawNotification: typing_extensions.TypeAlias = 'Notification | str'
NotificationSource: typing_extensions.TypeAlias = 'RawNotification | collections.abc.Iterable[RawNotification] | None'


class Notification:
    """Details on a notification to show."""

    __slots__ = ('__caption', '__hashsum', '__message', '__tags')

    def __init__(
            self,
            message: str,
            caption: str = 'Notification',
            tags: collections.abc.Iterable[str] | str = (),
    ) -> None:
        """Initialize new instance of :class:`~Notification`."""
        if isinstance(tags, str) or (not isinstance(tags, collections.abc.Iterable)):
            tags = (str(tags),)
        else:
            tags = tuple(sorted(map(str, tags)))

        self.__caption: typing.Final[str] = str(caption)
        self.__message: typing.Final[str] = str(message)
        self.__tags: typing.Final[tuple[str, ...]] = tags
        self.__hashsum: typing.Final[int] = hash((self.__caption, self.__message) + tags)

    @property
    def caption(self) -> str:  # noqa: D401
        """Caption for the notification window."""
        return self.__caption

    @property
    def message(self) -> str:  # noqa: D401
        """Content of the notification."""
        return self.__message

    @property
    def tags(self) -> tuple[str, ...]:  # noqa: D401
        """Extra tags for the notification."""
        return self.__tags

    def __eq__(self, other: typing.Any) -> bool:
        """Check if instance is equal to another one."""
        if isinstance(other, Notification):
            return (self.caption == other.caption) and (self.message == other.message) and (self.tags == other.tags)
        return NotImplemented

    def __hash__(self) -> int:
        """Retrieve hash sum of an instance."""
        return self.__hashsum

    def __repr__(self) -> str:
        """Prepare string representation of an instance."""
        return f'{type(self).__name__}(message={self.__message!r}, caption={self.__caption!r}, tags={self.__tags!r})'

    def __str__(self) -> str:
        """Prepare string representation of an instance."""
        return self.__message


class NotificationCollection(typing.MutableSequence[Notification]):
    """Collection of :class:`~Notification` instances."""

    __slots__ = ('__content',)

    def __init__(self, content: collections.abc.Iterable[Notification] = ()) -> None:
        """Initialize new instance of :class:`~NotificationCollection`."""
        self.__content: typing.Final[list[Notification]] = list(content)

    @staticmethod
    def collect(
            message: NotificationSource,
            caption: str = 'Notification',
            tags: str | collections.abc.Sequence[str] = (),
    ) -> NotificationCollection:
        """
        Collect notifications from a :class:`~Notification` compatible source.

        :param message: Message text or source (existing :class:`~Notification` or list of them).
        :param caption: Default caption to apply to non :class:`~Notification` source items.
        :param tags: Default tags to apply to non :class:`~Notification` source items.
        """
        result: list[Notification] = []

        if message is None:
            message = []
        elif isinstance(message, (Notification, str)) or (not isinstance(message, collections.abc.Iterable)):
            message = [message]

        item: Notification | str
        for item in message:
            if not isinstance(item, Notification):
                item = Notification(caption=caption, message=item, tags=tags)
            result.append(item)

        return NotificationCollection(result)

    def exclude(self, items: collections.abc.Container[Notification]) -> NotificationCollection:
        """Prepare a new :code:`~NotificationCollection` from the current content except :code:`items`."""
        return NotificationCollection(
            item
            for item in self.__content
            if item not in items
        )

    def insert(self, index: int, value: Notification) -> None:
        """Insert new :code:`value` at :code:`index`."""
        self.__content.insert(index, value)

    def only(
            self,
            message: str | None = None,
            caption: str | None = None,
            tags: collections.abc.Iterable[str] | str | None = None,
            strict: bool = False,
    ) -> NotificationCollection:
        """
        Filter current collection to only include notifications with particular properties.

        :param message: Expected value of a :attr:`~Notification.message`.
        :param caption: Expected value of a :attr:`~Notification.caption`.
        :param tags: Expected subset of :attr:`~Notification.tags`.
        :param strict: Use :code:`tags` for a full comparison instead of just subset.
        """
        conditions: list[collections.abc.Callable[[Notification], bool]] = []

        if message is not None:
            conditions.append(lambda item: item.message == message)

        if caption is not None:
            conditions.append(lambda item: item.caption == caption)

        if tags is not None:
            if isinstance(tags, str) or (not isinstance(tags, collections.abc.Iterable)):
                tags = [tags]
            if strict:
                tags = tuple(sorted(map(str, tags)))
                conditions.append(lambda item: item.tags == tags)
            else:
                tags = set(map(str, tags))
                conditions.append(lambda item: typing.cast('set[str]', tags).issubset(item.tags))

        return NotificationCollection(
            item
            for item in self.__content
            if all(condition(item) for condition in conditions)
        )

    def unique(self) -> NotificationCollection:
        """Prepare a new :class:`~NotificationCollection` that only contains unique elements."""
        result: NotificationCollection = NotificationCollection()
        result.extend(item for item in self.__content if item not in result)
        return result

    def __delitem__(self, index: int | slice) -> None:
        """Remove item by its index."""
        del self.__content[index]

    @typing.overload
    def __getitem__(self, index: int) -> Notification:
        """Retrieve content from a collection."""

    @typing.overload
    def __getitem__(self, index: slice) -> NotificationCollection:
        """Retrieve content from a collection."""

    def __getitem__(self, index: int | slice) -> Notification | NotificationCollection:
        """Retrieve content from a collection."""
        if isinstance(index, slice):
            return NotificationCollection(self.__content[index])
        return self.__content[index]

    def __iter__(self) -> collections.abc.Iterator[Notification]:
        """Iterate through the content of a collection."""
        return iter(self.__content)

    def __len__(self) -> int:
        """Retrieve total number of added :class:`~Notification` instances."""
        return len(self.__content)

    def __repr__(self) -> str:
        """Prepare string representation of an instance."""
        return f'{type(self).__name__}({repr(self.__content)})'

    @typing.overload
    def __setitem__(self, index: int, value: Notification) -> None:
        """Update value of an item."""

    @typing.overload
    def __setitem__(self, index: slice, value: collections.abc.Iterable[Notification]) -> None:
        """Update value of an item."""

    def __setitem__(self, index: int | slice, value: Notification | collections.abc.Iterable[Notification]) -> None:
        """Update value of an item."""
        self.__content[index] = value  # type: ignore

    def __str__(self) -> str:
        """Prepare string representation of an instance."""
        return str(self.__content)


class NotificationListener(QObject):
    """Dynamic listener for notifications."""

    sig_notification = Signal(Notification)

    def push(self, notification: Notification) -> None:
        """Push new notification to the listener."""
        self.sig_notification.emit(notification)


class NotificationQueue:
    """Notification dialogs queue."""

    __slots__ = ('__history', '__listener', '__queue')

    def __init__(self) -> None:
        """Initialize new instance of :class:`~NotificationQueue`."""
        self.__history: typing.Final[set[Notification]] = set()
        self.__listener: NotificationListener | None = None
        self.__queue: typing.Final[collections.deque[Notification]] = collections.deque()

    def attach(self, listener: NotificationListener) -> None:
        """
        Attach listener to the queue.

        If there are any notifications pending - all of them will be dumped into the :code:`listener`.
        """
        self.__listener = listener
        while self.__queue:
            self.__listener.push(self.__queue.popleft())

    def push(
            self,
            message: NotificationSource,
            caption: str = 'Notification',
            tags: collections.abc.Sequence[str] | str = (),
            *,
            once: bool = False,
            unique: bool = True,
    ) -> NotificationCollection:
        """
        Push new notification to the queue.

        :param message: Message text or source (existing :class:`~Notification` or list of them).
        :param caption: Default caption to apply to non :class:`~Notification` source items.
        :param tags: Default tags to apply to non :class:`~Notification` source items.
        :param once: Show each notification only once per application session.
        :param unique: Show each notification only once per :meth:`~NotificationQueue.push` request. May also hide
                       duplicates until the listener is attached via :meth:`~NotificationQueue.attach`.
        """
        result: NotificationCollection = NotificationCollection.collect(message=message, caption=caption, tags=tags)
        delta: NotificationCollection = result

        if unique or once:
            delta = delta.unique()
        if once:
            delta = delta.exclude(self.__history)
            self.__history.update(delta)

        if self.__listener is None:
            if unique:
                delta = delta.exclude(self.__queue)
            self.__queue.extend(delta)
        else:
            for raw in delta:
                self.__listener.push(raw)

        return result


NOTIFICATION_QUEUE: typing.Final[NotificationQueue] = NotificationQueue()



# -*- coding: utf8 -*-

"""Primary bases for telemetry processing."""

from __future__ import annotations

__all__ = ['Event', 'Context', 'Provider', 'Pool', 'Analytics']

import abc
import collections.abc
import datetime
import queue
import typing
import uuid

from anaconda_navigator.utils import workers

if typing.TYPE_CHECKING:
    import typing_extensions


KeyT = typing.TypeVar('KeyT', bound=collections.abc.Hashable)
ValueT = typing.TypeVar('ValueT')

EventProperty: typing_extensions.TypeAlias = 'str | int'
EventPropertyCollection: typing_extensions.TypeAlias = 'collections.abc.Mapping[str, EventProperty]'
MappingSource: typing_extensions.TypeAlias = (
    'collections.abc.Mapping[KeyT, ValueT] | collections.abc.Iterable[tuple[KeyT, ValueT]]'
)


def generate_identifier() -> str:
    """Generate random identifier string."""
    return f'{uuid.uuid4().hex}{uuid.uuid4().hex}'


class Event:
    """Telemetry event to track."""

    __slots__ = ('__key', '__name', '__properties', '__timestamp', '__user_properties')

    def __init__(
            self,
            name: str,
            properties: MappingSource[str, EventProperty] = (),  # pylint: disable=invalid-sequence-index
            *,
            key: str | None = None,
            timestamp: datetime.datetime | None = None,
            user_properties: MappingSource[str, EventProperty] = (),  # pylint: disable=invalid-sequence-index
    ) -> None:
        """Initialize new instance of a :class:`~Event`."""
        self.__key: typing.Final[str] = str(key or uuid.uuid4().hex)
        self.__name: typing.Final[str] = str(name)
        self.__properties: typing.Final[EventPropertyCollection] = dict(properties)
        self.__timestamp: typing.Final[datetime.datetime] = timestamp or datetime.datetime.now(datetime.timezone.utc)
        self.__user_properties: typing.Final[EventPropertyCollection] = dict(user_properties)

    @property
    def key(self) -> str:  # noqa: D401
        """Unique identifier of the event."""
        return self.__key

    @property
    def name(self) -> str:  # noqa: D401
        """Name of the event."""
        return self.__name

    @property
    def properties(self) -> EventPropertyCollection:  # noqa: D401
        """Extra properties of the event."""
        return self.__properties

    @property
    def timestamp(self) -> datetime.datetime:
        """When event is happened."""
        return self.__timestamp

    @property
    def user_properties(self) -> EventPropertyCollection:  # noqa: D401
        """Extra properties of the user triggered the event."""
        return self.__user_properties


class Context:
    """
    Common context for telemetry processing.

    :param user_identity: Unique identifier of the current user.
    :param session_identity: Unique identifier of the current session.
    """

    __slots__ = ('__session_identity', '__user_identity')

    def __init__(
            self,
            session_identity: str | None = None,
            user_identity: str | None = None,
    ) -> None:
        """Initialize new instance of a :class:`~Context`."""
        if not session_identity:
            session_identity = generate_identifier()
        if not user_identity:
            user_identity = generate_identifier()

        self.__session_identity: typing.Final[str] = str(session_identity)
        self.__user_identity: typing.Final[str] = str(user_identity)

    @property
    def session_identity(self) -> str:  # noqa: D401
        """Unique identifier of the current session."""
        return self.__session_identity

    @property
    def user_identity(self) -> str:  # noqa: D401
        """Unique identifier of the current user."""
        return self.__user_identity


class Provider(metaclass=abc.ABCMeta):
    """
    Integration with telemetry service.

    :param alias: Unique name of the provider.

                  Used to identify providers when events are sent to them.

                  If multiple providers use the same name - each event will be processed by at most one of them. This
                  might be useful for chain of responsibility implementation, but otherwise be careful not to lose
                  collected data.

    :param context: Context with primary environment details.
    """

    __slots__ = ('__alias', '__context')

    def __init__(self, alias: str, context: Context) -> None:
        """Initialize new instance of a :class:`Provider`."""
        self.__alias: typing.Final[str] = alias
        self.__context: typing.Final[Context] = context

    @property
    def alias(self) -> str:  # noqa: D401
        """Unique name of the provider."""
        return self.__alias

    @property
    def context(self) -> Context:  # noqa: D401
        """Context with primary environment details."""
        return self.__context

    @abc.abstractmethod
    def close(self) -> None:
        """Close provider."""

    @abc.abstractmethod
    def process(self, events: collections.abc.MutableSequence[Event]) -> None:
        """
        Process all pending :code:`events`.

        Processed events must be removed from :code:`events`.
        """


class Pool(metaclass=abc.ABCMeta):
    """Storage and manager for telemetry events."""

    __slots__ = ()

    @abc.abstractmethod
    def pending(self, provider: str) -> collections.abc.MutableSequence[Event]:
        """
        Retrieve all currently pending events for :code:`provider`.

        It must be a mutable sequence that provider can modify, and those changes must be applied to the pool. It is
        required for providers to remove events that are already processed and keep only those that are yet to process.
        """

    @abc.abstractmethod
    def push(self, event: Event) -> None:
        """Push new :code:`event` to the pool."""

    @abc.abstractmethod
    def register(self, providers: collections.abc.Iterable[str]) -> None:
        """Register :code:`providers` in the pool for it to be aware of them."""


class Analytics:
    """
    Core component for telemetry management.

    :param providers: Provides to process telemetry with.
    :param pool: Storage for telemetry events.
    """

    __slots__ = ('__pool', '__providers', '__queue')

    def __init__(
            self,
            providers: collections.abc.Iterable[Provider],
            pool: Pool,
    ) -> None:
        """Initialize new instance of an :class:`~Analytics`."""
        self.__pool: typing.Final[Pool] = pool
        self.__providers: typing.Final[collections.abc.Sequence[Provider]] = tuple(providers)
        self.__queue: typing.Final[queue.Queue[Event | None]] = queue.Queue()

        self.__pool.register(provider.alias for provider in self.__providers)

        workers.Task(self.__run, workers.AddCancelCallback(self.stop)).worker().start()

    @property
    def providers(self) -> collections.abc.Sequence[Provider]:  # noqa: D401
        """List of provides used to process events."""
        return self.__providers

    @typing.overload
    def event(self, name: Event) -> None:
        """Push new telemetry event."""

    @typing.overload
    def event(
            self,
            name: str,
            properties: MappingSource[str, EventProperty] = (),  # pylint: disable=invalid-sequence-index
            *,
            key: str | None = None,
            timestamp: datetime.datetime | None = None,
            user_properties: MappingSource[str, EventProperty] = (),  # pylint: disable=invalid-sequence-index
    ) -> None:
        """Push new telemetry event."""

    def event(
            self,
            name: str | Event,
            properties: MappingSource[str, EventProperty] = (),  # pylint: disable=invalid-sequence-index
            *,
            key: str | None = None,
            timestamp: datetime.datetime | None = None,
            user_properties: MappingSource[str, EventProperty] = (),  # pylint: disable=invalid-sequence-index
    ) -> None:
        """Push new telemetry event."""
        if not isinstance(name, Event):
            name = Event(
                key=key,
                name=name,
                properties=properties,
                timestamp=timestamp,
                user_properties=user_properties,
            )

        self.__queue.put(name)

    def stop(self) -> None:
        """Stop processing telemetry."""
        self.__queue.put(None)

    def __run(self) -> None:
        """Primary thread used to process analytics."""
        event: Event | None
        provider: Provider
        while event := self.__queue.get():
            self.__pool.push(event)
            for provider in self.providers:
                provider.process(self.__pool.pending(provider.alias))

        for provider in self.providers:
            provider.close()
